@@ -2,7 +2,7 @@
 """ PolymorphicManager
     Please see README.rst or DOCS.rst or http://chrisglass.github.com/django_polymorphic/
 """
-
+import warnings
 from django.db import models
 from polymorphic.query import PolymorphicQuerySet
 
@@ -14,17 +14,23 @@ class PolymorphicManager(models.Manager):
     Usually not explicitly needed, except if a custom manager or
     a custom queryset class is to be used.
     """
+    # Tell Django that related fields also need to use this manager:
     use_for_related_fields = True
+    queryset_class = PolymorphicQuerySet
 
     def __init__(self, queryset_class=None, *args, **kwrags):
-        if not queryset_class:
-            self.queryset_class = PolymorphicQuerySet
-        else:
+        # Up till polymorphic 0.4, the queryset class could be specified as parameter to __init__.
+        # However, this doesn't work for related managers which instantiate a new version of this class.
+        # Hence, for custom managers the new default is using the 'queryset_class' attribute at class level instead.
+        if queryset_class:
+            warnings.warn("Using PolymorphicManager(queryset_class=..) is deprecated; override the queryset_class attribute instead", DeprecationWarning)
+            # For backwards compatibility, still allow the parameter:
             self.queryset_class = queryset_class
+
         super(PolymorphicManager, self).__init__(*args, **kwrags)
 
     def get_query_set(self):
-        return self.queryset_class(self.model)
+        return self.queryset_class(self.model, using=self._db)
 
     # Proxy all unknown method calls to the queryset, so that its members are
     # directly accessible as PolymorphicModel.objects.*
